Tmax=1000;

net=readNet('bistability.net');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% first order model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('first-order model')
[mdyn1,x10]=closureDynamics(net,1,'derMatch','fun');

equations=cell(subsParameters(net,mdyn.dotMu.approxDotMu));
variables=cell(mdyn.Mu.sym);
solution=solve(equations{:},variables{:});

disp('Equilibrium point for X1:')
solution.mu_X1

options=odeset('RelTol',1e-6,'AbsTol',1e-6,'Stats','on');
[t1,mu1]=ode23s(@(t,x)fun(x),[0,Tmax],x10,options);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% second order model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('second-order model')
[mdyn2,x20]=closureDynamics(net,2,'derMatch','fun');

options=odeset('RelTol',1e-6,'AbsTol',1e-2,'Stats','on');
[t2,mu2]=ode23s(@(t,x)fun(x),[0,Tmax],x20,options);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% third order model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('third-order model')
[mdyn3,x30]=closureDynamics(net,3,'derMatch','fun');

options=odeset('RelTol',1e-4,'AbsTol',1e-2,'Stats','on');
[t3,mu3]=ode23s(@(t,x)fun(x),[0,Tmax],x30,options);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% plots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(1);clf
subplot(1,3,1)
[h1,av1,std1]=plotCMoments(net,mdyn1,t1,mu1,'X1','b-');
hold on
[h2,av2,std2]=plotCMoments(net,mdyn2,t2,mu2,'X1','g-');
[h3,av3,std3]=plotCMoments(net,mdyn3,t3,mu3,'X1','r-');
hold off
legend([h1(1),h2(1),h3(1)],'1st order','2nd order','3rd order','location','best');
grid on
title(sprintf('Moment truncation: E[X1]=%.3g, Std[X1]=%.3g (from 3rd trunc.)',av3(end),std3(end)))
drawnow

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SSA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
net2stochKit(net,'ProblemDefinition',[X10,X20,X30]);
%type ProblemDefinition.cpp

rc=system('make single stats');
if rc
  fprintf('\nFailed to make stochKit executable. Will not run Monte Simulations\n');
  return
end

system(sprintf('single %f single.txt',Tmax));
tx=load('single.txt');

subplot(1,3,2)
plot(tx(:,1),tx(:,2),'-',tx(:,1),tx(:,3:4),'--')
legend('X1','X2','X3','location','best')
title('Monte Carlo')
drawnow


system(sprintf('stats %f 10000 stats.txt',Tmax));
xfinal=load('stats.txt');

subplot(1,3,3)
hist(xfinal(:,1),20)
title(sprintf('Monte Carlo: E[X1]=%.3g, Std[X1]=%.3g',mean(xfinal(:,1)),std(xfinal(:,1))))
drawnow
